# -*- coding: utf-8 -*-
from __future__ import (
    print_function,
    division,
    unicode_literals,
    absolute_import
)
import compat.mock
from natsort.compat.py23 import PY_VERSION

# Use hypothesis if not on python 2.6.
if PY_VERSION > 2.6:
    use_hypothesis = True
    from hypothesis import assume, given, example
    from hypothesis.strategies import (
        sampled_from,
        integers,
        floats,
        tuples,
        lists,
        text,
        binary,
    )
# Otherwise mock these imports, because hypothesis
# is incompatible with python 2.6.
else:
    example = sampled_from = assume = given = floats = integers = \
        tuples = lists = text = binary = compat.mock.MagicMock()
    use_hypothesis = False



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

import pytest
from math import isnan
from natsort.ns_enum import ns
from natsort.utils import _parse_number_function
from compat.hypothesis import (
    assume,
    given,
    floats,
    integers,
    use_hypothesis,
)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_parse_number_function_makes_function_that_returns_tuple_example():
    assert _parse_number_function(0, '')(57) == ('', 57)
    assert _parse_number_function(0, '')(float('nan')) == ('', float('-inf'))
    assert _parse_number_function(ns.NANLAST, '')(float('nan')) == ('', float('+inf'))


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(floats() | integers())
def test_parse_number_function_makes_function_that_returns_tuple(x):
    assume(not isnan(x))
    assert _parse_number_function(0, '')(x) == ('', x)


def test_parse_number_function_with_PATH_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_function(ns.PATH, '')(57) == (('', 57), )


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(floats() | integers())
def test_parse_number_function_with_PATH_makes_function_that_returns_nested_tuple(x):
    assume(not isnan(x))
    assert _parse_number_function(ns.PATH, '')(x) == (('', x), )


def test_parse_number_function_with_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_function(ns.UNGROUPLETTERS | ns.LOCALE, '')(57) == (('',), ('', 57))


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(floats() | integers())
def test_parse_number_function_with_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple(x):
    assume(not isnan(x))
    assert _parse_number_function(ns.UNGROUPLETTERS | ns.LOCALE, '')(x) == (('',), ('', x))


def test_parse_number_function_with_PATH_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple_example():
    assert _parse_number_function(ns.PATH | ns.UNGROUPLETTERS | ns.LOCALE, '')(57) == ((('',), ('', 57)),)


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(floats() | integers())
def test_parse_number_function_with_PATH_UNGROUPLETTERS_LOCALE_makes_function_that_returns_nested_tuple(x):
    assume(not isnan(x))
    assert _parse_number_function(ns.PATH | ns.UNGROUPLETTERS | ns.LOCALE, '')(x) == ((('',), ('', x)),)



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""
from __future__ import unicode_literals

import pytest
from natsort.ns_enum import ns
from natsort.utils import _parse_bytes_function
from compat.hypothesis import (
    given,
    binary,
    use_hypothesis,
)


# Each test has an "example" version for demonstrative purposes,
# and a test that uses the hypothesis module.


def test_parse_bytes_function_makes_function_that_returns_tuple_example():
    assert _parse_bytes_function(0)(b'hello') == (b'hello', )


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(binary())
def test_parse_bytes_function_makes_function_that_returns_tuple(x):
    assert _parse_bytes_function(0)(x) == (x, )


def test_parse_bytes_function_with_IGNORECASE_makes_function_that_returns_tuple_with_lowercase_example():
    assert _parse_bytes_function(ns.IGNORECASE)(b'HelLo') == (b'hello', )


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(binary())
def test_parse_bytes_function_with_IGNORECASE_makes_function_that_returns_tuple_with_lowercase(x):
    assert _parse_bytes_function(ns.IGNORECASE)(x) == (x.lower(), )


def test_parse_bytes_function_with_PATH_makes_function_that_returns_nested_tuple_example():
    assert _parse_bytes_function(ns.PATH)(b'hello') == ((b'hello', ), )


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(binary())
def test_parse_bytes_function_with_PATH_makes_function_that_returns_nested_tuple(x):
    assert _parse_bytes_function(ns.PATH)(x) == ((x, ), )


def test_parse_bytes_function_with_PATH_and_IGNORECASE_makes_function_that_returns_nested_tuple_with_lowercase_example():
    assert _parse_bytes_function(ns.PATH | ns.IGNORECASE)(b'HelLo') == ((b'hello', ), )


@pytest.mark.skipif(not use_hypothesis, reason='requires python2.7 or greater')
@given(binary())
def test_parse_bytes_function_with_PATH_and_IGNORECASE_makes_function_that_returns_nested_tuple_with_lowercase(x):
    assert _parse_bytes_function(ns.PATH | ns.IGNORECASE)(x) == ((x.lower(), ), )



#! /usr/bin/env python

# Std. lib imports
import re
import sys
from os.path import join

# Non-std lib imports
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    """Custom command to run pytest on all code."""

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        if sys.version[:3] == '2.6':
            return pytest.main()
        else:
            return pytest.main(['--cov', 'natsort',
                                '--cov-report', 'term-missing',
                                '--flakes',
                                '--pep8',
                                # '-s',
                                # '--failed',
                                # '-v',
                                # 'test_natsort',
                                # 'README.rst',
                                # 'docs/source/intro.rst',
                                # 'docs/source/examples.rst',
                                ])


# Read the natsort.py file for the module version number
VERSIONFILE = join('natsort', '_version.py')
versionsearch = re.compile(r"^__version__ = ['\"]([^'\"]*)['\"]")
with open(VERSIONFILE, "rt") as fl:
    for line in fl:
        m = versionsearch.search(line)
        if m:
            VERSION = m.group(1)
            break
    else:
        s = "Unable to locate version string in {0}"
        raise RuntimeError(s.format(VERSIONFILE))

# Read in the documentation for the long_description
DESCRIPTION = 'Sort lists naturally'
try:
    with open('README.rst') as fl:
        LONG_DESCRIPTION = fl.read()
except IOError:
    LONG_DESCRIPTION = DESCRIPTION

# The argparse module was introduced in python 2.7 or python 3.2
REQUIRES = 'argparse' if sys.version[:3] in ('2.6', '3.0', '3.1') else ''

# Testing needs pytest, and mock if less than python 3.3
TESTS_REQUIRE = ['pytest', 'pytest-pep8', 'pytest-flakes',
                 'pytest-cov', 'pytest-cache', 'hypothesis']

if (sys.version.startswith('2') or
        (sys.version.startswith('3') and int(sys.version.split('.')[1]) < 3)):
    TESTS_REQUIRE.append('mock')
if (sys.version.startswith('2') or
        (sys.version.startswith('3') and int(sys.version.split('.')[1]) < 4)):
    TESTS_REQUIRE.append('pathlib')

# The setup parameters
setup(
    name='natsort',
    version=VERSION,
    author='Seth M. Morton',
    author_email='drtuba78@gmail.com',
    url='https://github.com/SethMMorton/natsort',
    license='MIT',
    install_requires=REQUIRES,
    packages=find_packages(exclude=['test*']),
    entry_points={'console_scripts': ['natsort = natsort.__main__:main']},
    tests_require=TESTS_REQUIRE,
    cmdclass={'test': PyTest},
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Financial and Insurance Industry',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Utilities',
        'Topic :: Text Processing',
    )
)
